@@ -1,8 +1,4 @@
 
-set_true <- function(x) rep(TRUE, length(x))
-
-set_na <- function(x) rep(NA, length(x))
-
 vent_flag <- function(x, val_var, ...) {
   x <- x[as.logical(get(val_var)), ]
   set(x, j = c(index_var(x), val_var),
@@ -37,40 +33,10 @@ force_type <- function(type) {
   }
 }
 
-eicu_body_weight <- function(x, val_var, weight_var, env, ...) {
-
-  do_calc <- function(rate, w1, w2) rate / fifelse(is.na(w1), w2, w1)
-
-  idc <- id_vars(x)
-
-  weight <- load_id("patient", env, cols = "admissionweight", id_var = idc)
-
-  x <- merge(x, weight, all.x = TRUE, by = idc)
-
-  force_num <- force_type("numeric")
-
-  x <- set(x, j = val_var, value = force_num(x[[val_var]]))
-  x <- set(x, j = weight_var, value = force_num(x[[weight_var]]))
-
-  x <- x[, c(val_var) := do_calc(get(val_var), get(weight_var),
-                                 get("admissionweight"))]
-
-  rm_cols(x, "admissionweight")
-}
-
-combine_date_time <- function(x, date_var, date_shift = hours(12L), ...) {
+combine_date_time <- function(x, time_var, date_shift = hours(12L), ...) {
   idx <- index_var(x)
-  x[, c(idx) := fifelse(is.na(get(idx)),
-                        get(date_var) + date_shift, get(idx))]
-}
-
-shift_all_date <- function(x, shift = hours(12L), ...) {
-  idx <- index_var(x)
-  x[, c(idx) := get(idx) + shift]
-}
-
-mimic_abx_shift_flag <- function(x, val_var, ...) {
-  transform_fun(set_true)(shift_all_date(x, hours(12L)), val_var)
+  x[, c(idx) := fifelse(is.na(get(time_var)),
+                        get(idx) + date_shift, get(time_var))]
 }
 
 mimic_sampling <- function(x, val_var, aux_time, ...) {
@@ -189,6 +155,12 @@ binary_op <- function(op, y) function(x) op(x, y)
 #' @export
 comp_na <- function(op, y) function(x) !is.na(x) & op(x, y)
 
+#' @param val Value to replace every element of x with
+#'
+#' @rdname callback_itm
+#' @export
+set_val <- function(val) function(x) rep(val, length(x))
+
 distribute_amount <- function(x, val_var, unit_var, end_var, ...) {
 
   hr  <- `units<-`(hours(1L), time_unit(x))
@@ -216,35 +188,27 @@ aggregate_fun <- function(fun, new_unit) {
   }
 }
 
-aggregate_amount <- function(x, val_var, unit_var, ...) {
-  res <- dt_gforce(set(x, j = unit_var, value = NULL), "sum")
-  res <- res[, c(unit_var) := "units"]
-  res
+mimic_age <- function(x) {
+  x <- as.double(x, units = "days") / -365
+  ifelse(x > 90, 90, x)
 }
 
-mimic_age <- function(x, val_var, ...) {
-  x <- set(x, j = val_var,
-    value = as.double(`units<-`(x[[val_var]], "days") / -365))
-  x <- set(x, i = which(x[[val_var]] > 90), j = val_var, value = 90)
-  x
-}
+eicu_age <- function(x) as.numeric(ifelse(x == "> 89", 90, x))
 
-eicu_age <- function(x, val_var, ...) {
-  x <- set(x, which(x[[val_var]] == "> 89"), j = val_var, value = 90)
-  x <- set(x, j = val_var, value = as.numeric(x[[val_var]]))
-  x
-}
+hirid_death <- function(x, val_var, sub_var, env, ...) {
 
-hirid_death <- function(x, val_var, sub_var, ...) {
+  dis <- "discharge_status"
 
   idx <- index_var(x)
   idc <- id_vars(x)
 
-  tmp <- dt_gforce(x, "last", by = c(idc, sub_var), vars = c(idx, val_var))
-  tmp <- tmp[, list(max(get(idx)), any(get(val_var) <= 40)), by = c(idc)]
-  tmp <- rename_cols(tmp, c(idc, idx, val_var), by_ref = TRUE)
+  res <- dt_gforce(x, "last", by = idc, vars = idx)
+  tmp <- load_id(env[["general"]], cols = dis)
 
-  as_ts_tbl(tmp, index_var = idx, interval = interval(x), by_ref = TRUE)
+  res <- merge(res, tmp[is_true(get(dis) == "dead"), ])
+  res <- res[, c(val_var, dis) := list(TRUE, NULL)]
+
+  res
 }
 
 #' @param map Named atomic vector used for mapping a set of values (the names
@@ -262,15 +226,15 @@ apply_map <- function(map) {
   }
 }
 
-#' @param rgx Regular expression(s) used for identifying observations based on
-#' their current unit of measurement
 #' @param fun Function(s) used for transforming matching values
 #' @param new Name(s) of transformed units
+#' @param rgx Regular expression(s) used for identifying observations based on
+#' their current unit of measurement, `NULL` means everything
 #' @param ignore_case Forwarded to [base::grep()]
 #'
 #' @rdname callback_itm
 #' @export
-convert_unit <- function(rgx, fun, new, ignore_case = TRUE, ...) {
+convert_unit <- function(fun, new, rgx = NULL, ignore_case = TRUE, ...) {
 
   chr_to_fun <- function(a) {
 
@@ -285,6 +249,10 @@ convert_unit <- function(rgx, fun, new, ignore_case = TRUE, ...) {
     fun <- list(fun)
   }
 
+  if (is.null(rgx)) {
+    rgx <- rep(list(NULL), length(fun))
+  }
+
   new <- lapply(new, chr_to_fun)
 
   assert_that(all_fun(fun, is.function))
@@ -295,9 +263,20 @@ convert_unit <- function(rgx, fun, new, ignore_case = TRUE, ...) {
   function(x, val_var, unit_var, ...) {
 
     for (i in seq_len(len)) {
-      rows <- do.call(grep, c(list(rgx[[i]], x[[unit_var]]), xtr))
-      vals <- list(fun[[i]](x[[val_var]][rows]), new[[i]](x[[unit_var]][rows]))
-      set(x, i = rows, j = c(val_var, unit_var), value = vals)
+
+      if (is.null(rgx[[i]])) {
+
+        vals <- list(fun[[i]](x[[val_var]]), new[[i]](x[[unit_var]]))
+        set(x, j = c(val_var, unit_var), value = vals)
+
+      } else {
+
+        rows <- do.call(grep, c(list(rgx[[i]], x[[unit_var]]), xtr))
+        vals <- list(fun[[i]](x[[val_var]][rows]),
+                     new[[i]](x[[unit_var]][rows]))
+
+        set(x, i = rows, j = c(val_var, unit_var), value = vals)
+      }
     }
 
     x
@@ -321,57 +300,95 @@ eicu_adx <- function(x, val_var, ...) {
   set(x, j = val_var, value = cats)
 }
 
-weight_env <- new.env()
+add_concept_env <- new.env()
 
 #' Internal item callback utilities
 #'
-#' The utility function `add_weight()` is exported for convenience when adding
-#' external datasets and integrating concepts such as vasopressor rates which
-#' rely on patient weight. For this to function, the newly added dataset must
-#' offer a `weight` concept. For performance reasons, the weight concept is
-#' internally cached, as this might be used unchanged many times, when loading
-#' several concepts that need to pull in patient weight data.
+#' The utility function `add_concept()` is exported for convenience when adding
+#' external datasets and integrating concepts that require other concepts.
+#' While this could be solves by defining a `rec_concpt`, in some scenarios
+#' this might not be ideal, as it might be only required that `itm`
+#' implementations for certain data sources require additional information.
+#' Examples for this include vasopressor rates which might rely on patient
+#' weight, and blood cell counts when expressed as ratio. For performance
+#' reasons, the pulled in concept is internally cached, as this might be used
+#' unchanged many times, when loading several concepts that need to pull in
+#' the given concept. Persistence of cache is session-level and therefore this
+#' utility is intended to be used somewhat sparingly.
 #'
 #' @param x Object in loading
 #' @param env Data source environment as available as `env` in callback
 #' functions
-#' @param weight_var String valued name of the newly added weight column
+#' @param concept String valued concept name that will be loaded from the
+#' default dictionary
+#' @param var_name String valued variable name
+#' @param aggregate Forwarded to [load_concepts()]
 #'
-#' @return An `id_tbl` or `ts_tbl` object, potentially modified by reference.
+#' @return A copy of `x` with the requested concept merged in.
 #'
 #' @rdname callback_int
 #' @keywords internal
 #' @export
-add_weight <- function(x, env, weight_var = "weight") {
+add_concept <- function(x, env, concept, var_name = concept,
+                        aggregate = NULL) {
 
-  assert_that(is_id_tbl(x), is_src_env(env), is.string(weight_var))
+  assert_that(is_id_tbl(x), is_src_env(env), is.string(concept),
+              is.string(var_name), is_disjoint(var_name, colnames(x)))
 
-  cache  <- paste(src_name(env), id_vars(x), sep = "_")
-  weight <- get0(cache, envir = weight_env, inherits = FALSE)
+  idtyp <- id_name_to_type(env, id_var(x))
+  inval <- interval(x)
+  cache <- digest(concept, src_name(env), idtyp, inval, aggregate)
 
-  if (is.null(weight)) {
-    weight <- load_concepts("weight", src_name(env), verbose = FALSE,
-                            id_type = id_name_to_type(env, id_var(x)))
-    assign(cache, weight, envir = weight_env)
+  res <- get0(cache, envir = add_concept_env, inherits = FALSE)
+
+  if (is.null(res)) {
+
+    res <- load_concepts(concept, src_name(env), aggregate = aggregate,
+                         verbose = FALSE, id_type = idtyp, interval = inval)
+
+    assign(cache, res, envir = add_concept_env)
   }
 
-  if (is_in(weight_var, colnames(x))) {
-    tmp_var <- new_names(c(colnames(x), colnames(weight)))
+  if (!identical(concept, var_name)) {
+    res <- rename_cols(res, var_name, concept)
+  }
+
+  merge(x, res, all.x = TRUE)
+}
+
+#' @rdname callback_int
+#' @keywords internal
+#' @export
+add_weight <- function(x, env, var_name = "weight") {
+
+  var_exists <- is_in(var_name, colnames(x))
+
+  if (var_exists) {
+    tmp_var <- new_names(x)
   } else {
-    tmp_var <- weight_var
+    tmp_var <- var_name
   }
 
-  weight <- rename_cols(weight, tmp_var, "weight")
-  res    <- merge(x, weight, by = id_var(x), all.x = TRUE)
+  x <- add_concept(x, env, "weight", tmp_var)
 
-  if (is_in(weight_var, colnames(x))) {
-    res <- res[, c(weight_var) := silent_as_num(get(weight_var))]
-    res <- res[, c(weight_var, tmp_var) := list(
-      fifelse(is.na(get(weight_var)), get(tmp_var), get(weight_var)), NULL
+  if (var_exists) {
+    x <- x[, c(var_name) := silent_as_num(get(var_name))]
+    x <- x[, c(var_name, tmp_var) := list(
+      fifelse(is.na(get(var_name)), get(tmp_var), get(var_name)), NULL
     )]
   }
 
-  res
+  x
+}
+
+blood_cell_ratio <- function(x, val_var, unit_var, env, ...) {
+
+  x <- add_concept(x, env, "wbc")
+  x <- x[, c(val_var, "wbc", unit_var) := list(
+    100 * get(val_var) / get("wbc"), NULL, "%"
+  )]
+
+  x
 }
 
 eicu_extract_unit <- function(x) {
@@ -387,19 +404,47 @@ sub_trans <- function(regex, repl) {
   function(x) sub(regex, repl, x, ignore.case = TRUE)
 }
 
-eicu_vaso_rate <- function(ml_to_mcg) {
+expand_intervals <- function(x, keep_vars = NULL, grp_var = NULL) {
 
-  assert_that(is.count(ml_to_mcg))
+  x <- create_intervals(x, c(id_vars(x), grp_var), overhang = hours(1L),
+                        max_len = hours(6L), end_var = "endtime")
+  x <- expand(x, index_var(x), "endtime",
+              keep_vars = c(id_vars(x), keep_vars))
+  x
+}
+
+mimic_rate_cv <- function(x, val_var, grp_var, unit_var, ...) {
+  expand_intervals(x, c(val_var, unit_var), grp_var)
+}
+
+mimic_rate_mv <- function(x, val_var, unit_var, stop_var, ...) {
+  expand(x, index_var(x), stop_var,
+         keep_vars = c(id_vars(x), val_var, unit_var))
+}
+
+mimic_kg_rate <- function(x, val_var, unit_var, env, ...) {
+
+  x <- add_weight(x, env, "weight")
+  x <- x[, c(val_var, unit_var) := list(
+    get(val_var) / get("weight"), sub("mcgmin", "mcg/kg/min", get(unit_var))
+  )]
+
+  x
+}
+
+eicu_rate_kg <- function(ml_to_mcg) {
+
+  assert_that(is_number(ml_to_mcg))
 
   function(x, sub_var, val_var, weight_var, env, ...) {
 
     fix_units <- convert_unit(
-      c("/hr$", "^mg/", "^units/", "^ml/", "^nanograms/", "^Unknown$", "^ml$"),
-      c(binary_op(`/`, 60), binary_op(`*`, 1000), set_na,
-        binary_op(`*`, ml_to_mcg), binary_op(`/`, 1000), set_na, set_na),
+      c(binary_op(`/`, 60), binary_op(`*`, 1000), set_val(NA),
+        binary_op(`*`, ml_to_mcg), binary_op(`/`, 1000), set_val(NA), set_val(NA)),
       c(sub_trans("/hr$", "/min"), sub_trans("^mg/", "mcg/"),
         "mcg/kg/min", sub_trans("^ml/", "mcg/"),
-        sub_trans("^nanograms/", "mcg/"), "mcg/kg/min", "mcg/kg/min")
+        sub_trans("^nanograms/", "mcg/"), "mcg/kg/min", "mcg/kg/min"),
+      c("/hr$", "^mg/", "^units/", "^ml/", "^nanograms/", "^Unknown$", "^ml$")
     )
 
     add_kg <- sub_trans("/", "/kg/")
@@ -415,11 +460,38 @@ eicu_vaso_rate <- function(ml_to_mcg) {
       )
     ]
 
-    fix_units(x, val_var, "unit_var")
+    x <- fix_units(x, val_var, "unit_var")
+    x <- expand_intervals(x, c(val_var, "unit_var"))
+
+    x
   }
 }
 
-hirid_vaso_rate <- function(x, val_var, unit_var, env, ...) {
+eicu_rate_units <- function(ml_to_mcg, mcg_to_units) {
+
+  assert_that(is_number(ml_to_mcg), is_number(mcg_to_units))
+
+  function(x, sub_var, val_var, env, ...) {
+
+    fix_units <- convert_unit(
+      c(binary_op(`/`, 60), binary_op(`*`, 1000), set_val(NA),
+        binary_op(`*`, ml_to_mcg), binary_op(`*`, mcg_to_units)),
+      c(sub_trans("/hr$", "/min"), sub_trans("^mg/", "mcg/"),
+        "units/min", sub_trans("^ml/", "mcg/"), sub_trans("^mcg/", "units/")),
+      c("/hr$", "^mg/", "/kg/", "^ml/", "^mcg/")
+    )
+
+    x <- x[, c(val_var) := silent_as_num(get(val_var))]
+    x <- x[, c("unit_var") := eicu_extract_unit(get(sub_var))]
+
+    x <- fix_units(x, val_var, "unit_var")
+    x <- expand_intervals(x, c(val_var, "unit_var"))
+
+    x
+  }
+}
+
+hirid_rate_kg <- function(x, val_var, unit_var, grp_var, env, ...) {
 
   x <- x[get(unit_var) == "mg",
     c(val_var, unit_var) := list(1000 * get(val_var), "\u00b5g")
@@ -434,14 +506,40 @@ hirid_vaso_rate <- function(x, val_var, unit_var, env, ...) {
                   unexpected units")
   }
 
-  x <- dt_gforce(x, "sum", vars = val_var)
+  x <- dt_gforce(x, "sum", by = c(meta_vars(x), grp_var), vars = val_var)
   x <- add_weight(x, env, "weight")
 
   frac <- 1 / as.double(interval(x), units = "mins")
 
-  x[, c(val_var, unit_var, "weight") := list(
+  x <- x[, c(val_var, unit_var, "weight") := list(
     frac * get(val_var) / get("weight"), "mcg/kg/min", NULL)
   ]
+
+  expand_intervals(x, val_var, grp_var)
+}
+
+hirid_rate <- function(x, val_var, unit_var, grp_var, env, ...) {
+
+  unit <- table(x[[unit_var]])
+  unit <- names(unit)[which.max(unit)]
+
+  old_row <- nrow(x)
+  x <- x[get(unit_var) == unit, ]
+  dif_row <- old_row - nrow(x)
+
+  if (dif_row > 0) {
+    msg_progress("lost {dif_row} ({prcnt(dif_row, old_row)}) entries due to
+                  unexpected units")
+  }
+
+  x <- dt_gforce(x, "sum", by = c(meta_vars(x), grp_var), vars = val_var)
+
+  frac <- 1 / as.double(interval(x), units = "mins")
+  unit <- paste(unit, "min", sep = "/")
+
+  x <- x[, c(val_var, unit_var) := list(frac * get(val_var), unit)]
+
+  expand_intervals(x, c(val_var, unit_var), grp_var)
 }
 
 mimic_dur_incv <- function(x, val_var, grp_var, ...) {
@@ -452,33 +550,70 @@ mimic_dur_inmv <- function(x, val_var, grp_var, stop_var, ...) {
   calc_dur(x, val_var, index_var(x), stop_var, grp_var)
 }
 
+aumc_rate_kg <- function(x, val_var, unit_var, rel_weight, rate_uom, stop_var,
+                         env, ...) {
+
+  mg_to_mcg <- convert_unit(binary_op(`*`, 1000), "mcg", "mg")
+  hr_to_min <- convert_unit(binary_op(`/`, 60),   "min", "uur")
+
+  res <- rm_na(x, c(unit_var, rate_uom), "any")
+  res <- mg_to_mcg(res, val_var, unit_var)
+  res <- hr_to_min(res, val_var, rate_uom)
+
+  res <- add_weight(res, env, "weight")
+
+  res <- res[!get(rel_weight), c(val_var) := get(val_var) / get("weight")]
+  res <- res[get(unit_var) == "\u00b5g", c(unit_var) := "mcg"]
+  res <- res[, c(unit_var) := paste(
+    get(unit_var), get(rate_uom), sep = "/kg/"
+  )]
+
+  expand(res, index_var(x), stop_var,
+         keep_vars = c(id_vars(x), val_var, unit_var))
+}
+
+aumc_rate_units <- function(mcg_to_units) {
+
+  assert_that(is_number(mcg_to_units))
+
+  function(x, val_var, unit_var, rate_uom, stop_var, env, ...) {
+
+    frac <- 1 / as.double(interval(x), units = "mins")
+
+    to_units <- convert_unit(
+      c(identity, binary_op(`*`, 1000), binary_op(`*`, mcg_to_units)),
+      c("mcg", "mcg", "units"),
+      c("\u00b5g", "mg", "mcg")
+    )
+
+    to_min <- convert_unit(
+      c(binary_op(`/`, 24), binary_op(`/`, 60)),
+      c("uur", "min"),
+      c("dag", "uur")
+    )
+
+    x <- x[is.na(get(rate_uom)), c(val_var, rate_uom) := list(
+      sum(get(val_var)) * frac, "min"), by = c(meta_vars(x))
+    ]
+
+    x <- to_units(to_min(x, val_var, rate_uom), val_var, unit_var)
+    x <- x[, c(unit_var) := paste(get(unit_var), get(rate_uom), sep = "/")]
+
+    expand(x, index_var(x), stop_var,
+           keep_vars = c(id_vars(x), val_var, unit_var))
+  }
+}
+
 eicu_duration <- function(gap_length) {
 
   assert_that(is_interval(gap_length), is_scalar(gap_length))
 
   function(x, val_var, ...) {
 
-    time_diff <- function(x) x - data.table::shift(x)
+    x <- group_measurements(x, gap_length, "grp_var")
+    x <- calc_dur(x, val_var, index_var(x), index_var(x), "grp_var")
 
-    grp_calc  <- function(x) {
-      tmp <- rle(is_true(x <= gap_length))
-      val <- tmp[["values"]]
-      tmp[["values"]][val] <- seq_len(sum(val))
-      inverse.rle(tmp)
-    }
-
-    id_vars   <- id_vars(x)
-    index_var <- index_var(x)
-
-    if (interval(x) > gap_length) {
-      warn_ricu("splitting durations by gaps of length {format(gap_length)}
-                 using data with a time resolution of {format(interval(x))}")
-    }
-
-    x <- x[, c("time_diff") := time_diff(get(index_var)), by = c(id_vars)]
-    x <- x[, c("grp_var") := grp_calc(get("time_diff"))]
-
-    calc_dur(x, val_var, index_var, index_var, "grp_var")
+    x
   }
 }
 
@@ -486,6 +621,10 @@ hirid_duration <- function(x, val_var, grp_var, ...) {
   calc_dur(x, val_var, index_var(x), index_var(x), grp_var)
 }
 
+aumc_dur <- function(x, val_var, stop_var, grp_var, ...) {
+  calc_dur(x, val_var, index_var(x), stop_var, grp_var)
+}
+
 #' Used for determining vasopressor durations, `calc_dur()` will calculate
 #' durations by taking either per ID or per combination of ID and `grp_var`
 #' the minimum for `min_var` and the maximum of `max_var` and returning the
@@ -516,6 +655,31 @@ calc_dur <- function(x, val_var, min_var, max_var, grp_var = NULL) {
   res
 }
 
+#' Callback functions can be combined into an aggregated-callback function,
+#' which iterates over all functions passed as `...`, passing the result of
+#' one sub-callback function as argument `x` to the next.
+#'
+#' @param ... Functions which will be successively applied
+#'
+#' @rdname callback_int
+#' @keywords internal
+#' @export
+combine_callbacks <- function(...) {
+
+  funs <- list(...)
+
+  assert_that(all_fun(funs, is.function))
+
+  function(x, ...) {
+
+    for (fun in funs) {
+      x <- fun(x, ...)
+    }
+
+    x
+  }
+}
+
 hirid_urine <- function(x, val_var, unit_var, ...) {
 
   do_diff <- function(x) {
@@ -559,3 +723,41 @@ los_callback <- function(x, id_type, interval) {
 
   res
 }
+
+mimic_abx_presc <- function(x, id_type, interval) {
+
+  itm_cb <- function(x, val_var, ...) {
+
+    idx <- index_var(x)
+
+    x <- x[, c(idx, val_var) := list(get(idx) + mins(720L), TRUE)]
+    x
+  }
+
+  itm <- set_callback(x, itm_cb)
+  itm <- do.call(new_itm, c(itm, list(class = "rgx_itm")))
+  res <- do_itm_load(itm, "hadm", mins(1L))
+
+  res <- change_id(res, id_type, src_name(x), keep_old_id = FALSE,
+                   id_type = TRUE)
+
+  res <- do_callback(itm, res)
+  res <- change_interval(res, interval, by_ref = TRUE)
+
+  res
+}
+
+aumc_death <- function(x, val_var, ...) {
+
+  idx <- index_var(x)
+
+  x <- x[, c(val_var) := is_true(get(idx) - get(val_var) < hours(72L))]
+  x
+}
+
+aumc_bxs <- function(x, val_var, dir_var, ...) {
+  x <- x[get(dir_var) == "-", c(val_var) := -1L * get(val_var)]
+  x
+}
+
+aumc_rass <- function(x) as.integer(substr(x, 1L, 2L))
