@@ -212,7 +212,7 @@ id_win_helper.hirid_env <- function(x) {
 
   ind_fun <- function(id, index) {
 
-    tmp <- data.table::setDT(list(id = id, index = index))
+    tmp <- setDT(list(id = id, index = index))
 
     ind <- tmp[, .I[which.max(get("index"))], by = "id"][["V1"]]
 
@@ -227,11 +227,15 @@ id_win_helper.hirid_env <- function(x) {
   ids <- field(cfg, "id")
   sta <- field(cfg, "start")
 
-  obs <- as_src_tbl(x, "observations")
-  obs <- subset(obs, .env$ind_fun(.data$patientid, .data$datetime),
-                c("patientid", "datetime"), part_safe = TRUE)
+  obs <- load_src(
+    "observations", x, .env$ind_fun(.data$patientid, .data$datetime),
+    c("patientid", "datetime")
+  )
   obs <- obs[, list(datetime = max(get("datetime"))), by = "patientid"]
-  tbl <- as_src_tbl(x, field(cfg, "table"))[, c(ids, sta)]
+
+  tbl <- load_src(
+    field(cfg, "table"), x, cols = c(ids, sta)
+  )
 
   res <- merge(tbl, obs, by = ids)
   res <- res[, c(sta, "datetime") := lapply(.SD, as_dt_min, get(sta)),
@@ -240,6 +244,37 @@ id_win_helper.hirid_env <- function(x) {
   order_rename(res, ids, sta, "datetime")
 }
 
+#' @rdname data_utils
+#' @export
+id_win_helper.aumc_env <- function(x) {
+
+  cfg <- sort(as_id_cfg(x), decreasing = TRUE)
+
+  ids <- field(cfg, "id")
+  sta <- field(cfg, "start")
+  end <- field(cfg, "end")
+
+  tbl <- as_src_tbl(x, unique(field(cfg, "table")))
+
+  mis <- setdiff(sta, colnames(tbl))
+
+  assert_that(length(mis) >= 1L)
+
+  res <- load_src(tbl, cols = c(ids, intersect(sta, colnames(tbl)), end))
+
+  if (length(mis) > 0L) {
+    res[, c(mis) := 0L]
+  }
+
+  res <- res[, c(sta, end) := lapply(.SD, ms_as_mins), .SDcols = c(sta, end)]
+
+  res <- setcolorder(res, c(ids, sta, end))
+  res <- rename_cols(res, c(ids, paste0(ids, "_start"),
+                                 paste0(ids, "_end")), by_ref = TRUE)
+
+  as_id_tbl(res, ids[2L], by_ref = TRUE)
+}
+
 #' @export
 id_win_helper.default <- function(x) stop_generic(x, .Generic)
 
@@ -301,23 +336,18 @@ id_map_helper.src_env <- function(x, id_var, win_var) {
   map    <- id_windows(x)
   map_id <- id_vars(map)
 
-  if (identical(id_var, map_id)) {
-    ori <- NULL
-  } else {
-    ori <- paste0(id_var, "_start")
-  }
+  io_vars <- paste0(win_var, c("_start", "_end"))
 
-  inn <- paste0(win_var, "_start")
-  out <- paste0(win_var, "_end")
-
-  map <- map[, unique(c(id_var, win_var, inn, out, ori)), with = FALSE]
-
-  if (not_null(ori)) {
-    map <- map[, c(inn, out) := lapply(.SD, `-`, get(ori)),
-               .SDcols = c(inn, out)]
-    map <- rm_cols(map, ori, by_ref = TRUE)
+  if (!identical(id_var, map_id)) {
+    ori <- new_names(map)
+    map <- map[, c(ori) := get(paste0(id_var, "_start"))]
+    map <- map[, unique(c(id_var, win_var, io_vars, ori)), with = FALSE]
+    map <- map[, c(io_vars) := lapply(.SD, `-`, get(ori)),
+               .SDcols = c(io_vars)]
   }
 
+  kep <- setdiff(colnames(map), unique(c(id_var, win_var, io_vars)))
+  map <- rm_cols(map, kep, by_ref = TRUE)
   map <- unique(map)
 
   as_id_tbl(map, id_var, by_ref = TRUE)
@@ -334,21 +364,32 @@ id_map_helper.default <- function(x, ...) stop_generic(x, .Generic)
 #'
 #' @seealso change_id
 #'
-#' @param x Passed to [as_id_cfg()] and [as_src_env()]
-#' @param id_type Type of ID all returned times are relative to
-#' @param win_type Type of ID for which the in/out times is returned
-#' @param in_time,out_time column names of the returned in/out times
-#' @param interval The time interval used to discretize time stamps with,
-#' specified as [base::difftime()] object
+#' @param x Data source (is coerced to `src_env` using `as_src_env()`)
 #'
 #' @return An `id_tbl` containing the selected IDs and depending on values
 #' passed as `in_time` and `out_time`, start and end times of the ID passed as
 #' `win_var`.
 #'
+#' @rdname stay_windows
 #' @export
-stay_windows <- function(x, id_type = "icustay", win_type = id_type,
-                         in_time = "start", out_time = "end",
-                         interval = hours(1L)) {
+stay_windows <- function(x, ...) UseMethod("stay_windows", x)
+
+#' @param id_type Type of ID all returned times are relative to
+#' @param win_type Type of ID for which the in/out times is returned
+#' @param in_time,out_time column names of the returned in/out times
+#' @param interval The time interval used to discretize time stamps with,
+#' specified as [base::difftime()] object
+#' @param patient_ids Patient IDs used to subset the result
+#' @param ... Generic consistency
+#'
+#' @rdname stay_windows
+#' @export
+stay_windows.src_env <- function(x, id_type = "icustay", win_type = id_type,
+                                 in_time = "start", out_time = "end",
+                                 interval = hours(1L), patient_ids = NULL,
+                                 ...) {
+
+  warn_dots(...)
 
   assert_that(is_interval(interval))
 
@@ -362,9 +403,44 @@ stay_windows <- function(x, id_type = "icustay", win_type = id_type,
                .SDcols = c(in_time, out_time)]
   }
 
-  res
+  merge_patid(res, patient_ids)
 }
 
+#' @rdname stay_windows
+#' @export
+stay_windows.character <- function(x, ...) stay_windows(as.list(x), ...)
+
+#' @rdname stay_windows
+#' @export
+stay_windows.list <- function(x, ..., patient_ids = NULL) {
+
+  load_one <- function(x, ...) {
+
+    src <- src_name(x)
+    res <- stay_windows(x, ..., patient_ids = patient_ids[[src]])
+
+    if (mulit_src) {
+      res <- add_src_col(res, src)
+    }
+
+    res
+
+  }
+
+  x <- lapply(x, as_src_env)
+
+  srcs <- unique(chr_ply(x, src_name))
+  mulit_src <- length(srcs) > 1L
+
+  patient_ids <- split_patid(patient_ids, srcs)
+
+  rbind_lst(lapply(x, load_one, ...))
+}
+
+#' @rdname stay_windows
+#' @export
+stay_windows.default <- function(x, ...) stay_windows(as_src_env(x), ...)
+
 #' Switch between id types
 #'
 #' ICU datasets such as MIMIC-III or eICU typically represent patients by
@@ -388,7 +464,7 @@ stay_windows <- function(x, id_type = "icustay", win_type = id_type,
 #' example, we request for `mimic_demo`, with ICU stay IDs as source and
 #' hospital admissions as destination IDs.
 #'
-#' ```{r}
+#' ```{r, eval = is_data_avail("mimic_demo")}
 #' id_map_helper(mimic_demo, "icustay_id", "hadm_id")
 #' ```
 #'
@@ -442,6 +518,8 @@ stay_windows <- function(x, id_type = "icustay", win_type = id_type,
 #' @param ... Passed to `upgrade_id()`/`downgrade_id()`
 #' @param keep_old_id Logical flag indicating whether to keep the previous ID
 #' column
+#' @param id_type Logical flag indicating whether `target_id` is specified as
+#' ID name (e.g. `icustay_id` on MIMIC) or ID type (e.g. `icustay`)
 #'
 #' @return An object of the same type as `x` with modified IDs.
 #'
@@ -458,19 +536,27 @@ stay_windows <- function(x, id_type = "icustay", win_type = id_type,
 #' @rdname change_id
 #' @export
 #'
-change_id <- function(x, target_id, src, ..., keep_old_id = TRUE) {
+change_id <- function(x, target_id, src, ..., keep_old_id = TRUE,
+                      id_type = FALSE) {
 
   assert_that(is.string(target_id), is.flag(keep_old_id))
 
+  id_cfg <- as_id_cfg(src)
+
+  if (isTRUE(id_type)) {
+    target_id <- id_type_to_name(id_cfg, target_id)
+  }
+
   orig_id <- id_var(x)
 
   if (identical(orig_id, target_id)) {
     return(x)
   }
 
-  id_cfg <- as_id_cfg(src)
-
   opt <- id_var_opts(id_cfg)
+
+  assert_that(is_in(orig_id, opt), is_in(target_id, opt))
+
   ori <- id_cfg[orig_id   == opt]
   fin <- id_cfg[target_id == opt]
 
