@@ -33,8 +33,8 @@ col_spec_map <- function(type) {
   switch(type,
     bool = list(spec = "col_logical"),
     int2 = ,
-    int4 = ,
-    int8 = list(spec = "col_integer"),
+    int4 = list(spec = "col_integer"),
+    int8 = ,
     numeric = ,
     float8 = list(spec = "col_double"),
     bpchar = ,
@@ -293,6 +293,10 @@ eicu_tbl_cfg <- function(info, is_demo = FALSE) {
 
 mimic_tbl_cfg <- function(info, is_demo = FALSE) {
 
+  find_entry <- function(x, what, name) {
+    which(vapply(x, `[[`, character(1L), what) == name)
+  }
+
   files <- c("ADMISSIONS.csv.gz",
              "CALLOUT.csv.gz",
              "CAREGIVERS.csv.gz",
@@ -466,6 +470,16 @@ mimic_tbl_cfg <- function(info, is_demo = FALSE) {
       )
       x
     })
+
+    icd_diag <- find_entry(info, "table_name", "d_icd_diagnoses")
+    info[[icd_diag]]$num_rows <- 14567L
+
+    icd_proc <- find_entry(info, "table_name", "d_icd_procedures")
+    info[[icd_proc]]$num_rows <- 3882L
+
+    note <- find_entry(info, "table_name", "noteevents")
+    date <- find_entry(info[[note]]$cols, "name", "chartdate")
+    info[[note]]$cols[[date]]$format <- "%Y-%m-%d"
   }
 
   time_vars <- lapply(info, function(x) {
@@ -487,7 +501,8 @@ hirid_tbl_cfg <- function() {
       admissiontime = list(spec = "col_datetime",
                            format = "%Y-%m-%d %H:%M:%S"),
       sex = list(spec = "col_character"),
-      age = list(spec = "col_integer")
+      age = list(spec = "col_integer"),
+      discharge_status = list(spec = "col_character")
     ),
     observations = list(
       patientid = list(spec = "col_integer"),
@@ -536,7 +551,7 @@ hirid_tbl_cfg <- function() {
       "observation_tables", "csv", paste0("part-", 0L:249L, ".csv")
     ),
     ordinal = "ordinal_vars_ref.csv",
-    pharma= file.path(
+    pharma = file.path(
       "pharma_records", "csv", paste0("part-", 0L:249L, ".csv")
     ),
     variables = "hirid_variable_reference.csv"
@@ -607,6 +622,197 @@ hirid_tbl_cfg <- function() {
   as_tbl_spec(files, defaults, time_vars, info, part)
 }
 
+aumc_tbl_cfg <- function() {
+
+  info <- list(
+    admissions = list(
+      patientid = list(spec = "col_integer"),
+      admissionid = list(spec = "col_integer"),
+      admissioncount = list(spec = "col_integer"),
+      location = list(spec = "col_character"),
+      urgency = list(spec = "col_logical"),
+      origin = list(spec = "col_character"),
+      admittedat = list(spec = "col_double"),
+      admissionyeargroup = list(spec = "col_character"),
+      dischargedat = list(spec = "col_double"),
+      lengthofstay = list(spec = "col_integer"),
+      destination = list(spec = "col_character"),
+      gender = list(spec = "col_character"),
+      agegroup = list(spec = "col_character"),
+      dateofdeath = list(spec = "col_double"),
+      weightgroup = list(spec = "col_character"),
+      weightsource = list(spec = "col_character"),
+      specialty = list(spec = "col_character")
+    ),
+    drugitems = list(
+      admissionid = list(spec = "col_integer"),
+      orderid = list(spec = "col_integer"),
+      ordercategoryid = list(spec = "col_integer"),
+      ordercategory = list(spec = "col_character"),
+      itemid = list(spec = "col_integer"),
+      item = list(spec = "col_character"),
+      isadditive = list(spec = "col_logical"),
+      isconditional = list(spec = "col_logical"),
+      rate = list(spec = "col_double"),
+      rateunit = list(spec = "col_character"),
+      rateunitid = list(spec = "col_integer"),
+      ratetimeunitid = list(spec = "col_integer"),
+      doserateperkg = list(spec = "col_logical"),
+      dose = list(spec = "col_double"),
+      doseunit = list(spec = "col_character"),
+      doserateunit = list(spec = "col_character"),
+      doseunitid = list(spec = "col_integer"),
+      doserateunitid = list(spec = "col_integer"),
+      administered = list(spec = "col_double"),
+      administeredunit = list(spec = "col_character"),
+      administeredunitid = list(spec = "col_integer"),
+      action = list(spec = "col_character"),
+      start = list(spec = "col_double"),
+      stop = list(spec = "col_double"),
+      duration = list(spec = "col_integer"),
+      solutionitemid = list(spec = "col_integer"),
+      solutionitem = list(spec = "col_character"),
+      solutionadministered = list(spec = "col_double"),
+      solutionadministeredunit = list(spec = "col_character"),
+      fluidin = list(spec = "col_double"),
+      iscontinuous = list(spec = "col_logical")
+    ),
+    freetextitems = list(
+      admissionid = list(spec = "col_integer"),
+      itemid = list(spec = "col_integer"),
+      item = list(spec = "col_character"),
+      value = list(spec = "col_character"),
+      comment = list(spec = "col_character"),
+      measuredat = list(spec = "col_double"),
+      registeredat = list(spec = "col_double"),
+      registeredby = list(spec = "col_character"),
+      updatedat = list(spec = "col_double"),
+      updatedby = list(spec = "col_character"),
+      islabresult = list(spec = "col_logical")
+    ),
+    listitems = list(
+      admissionid = list(spec = "col_integer"),
+      itemid = list(spec = "col_integer"),
+      item = list(spec = "col_character"),
+      valueid = list(spec = "col_integer"),
+      value = list(spec = "col_character"),
+      measuredat = list(spec = "col_double"),
+      registeredat = list(spec = "col_double"),
+      registeredby = list(spec = "col_character"),
+      updatedat = list(spec = "col_double"),
+      updatedby = list(spec = "col_character"),
+      islabresult = list(spec = "col_logical")
+    ),
+    numericitems = list(
+      admissionid = list(spec = "col_integer"),
+      itemid = list(spec = "col_integer"),
+      item = list(spec = "col_character"),
+      tag = list(spec = "col_character"),
+      value = list(spec = "col_double"),
+      unitid = list(spec = "col_integer"),
+      unit = list(spec = "col_character"),
+      comment = list(spec = "col_character"),
+      measuredat = list(spec = "col_double"),
+      registeredat = list(spec = "col_double"),
+      registeredby = list(spec = "col_character"),
+      updatedat = list(spec = "col_double"),
+      updatedby = list(spec = "col_character"),
+      islabresult = list(spec = "col_logical"),
+      fluidout = list(spec = "col_double")
+    ),
+    procedureorderitems = list(
+      admissionid = list(spec = "col_integer"),
+      orderid = list(spec = "col_integer"),
+      ordercategoryid = list(spec = "col_integer"),
+      ordercategoryname = list(spec = "col_character"),
+      itemid = list(spec = "col_integer"),
+      item = list(spec = "col_character"),
+      registeredat = list(spec = "col_double"),
+      registeredby = list(spec = "col_character")
+    ),
+    processitems = list(
+      admissionid = list(spec = "col_integer"),
+      itemid = list(spec = "col_integer"),
+      item = list(spec = "col_character"),
+      start = list(spec = "col_double"),
+      stop = list(spec = "col_double"),
+      duration = list(spec = "col_integer")
+    )
+  )
+
+  tables <- names(info)
+
+  cols <- lapply(info, function(tbl) {
+    Map(function(name, spec) c(list(name = name), spec), names(tbl), tbl)
+  })
+
+  defaults <- list(
+    admissions = list(
+      index_var = "admittedat",
+      time_vars = c("admittedat", "dischargedat", "dateofdeath")
+    ),
+    drugitems = list(
+      index_var = "start",
+      val_var = "dose",
+      unit_var = "doseunit",
+      time_vars = c("start", "stop")
+    ),
+    freetextitems = list(
+      index_var = "measuredat",
+      id_var = "value",
+      time_vars = c("measuredat", "registeredat", "updatedat")
+    ),
+    listitems = list(
+      index_var = "measuredat",
+      val_var = "value",
+      time_vars = c("measuredat", "registeredat", "updatedat")
+    ),
+    numericitems = list(
+      index_var = "measuredat",
+      val_var = "value",
+      unit_var = "unit",
+      time_vars = c("measuredat", "registeredat", "updatedat")
+    ),
+    procedureorderitems = list(
+      index_var = "registeredat",
+      val_var = "item",
+      time_vars = "registeredat"
+    ),
+    processitems = list(
+      index_var = "start",
+      val_var = "item",
+      time_vars = c("start", "stop")
+    )
+  )
+
+  n_row <- list(
+    admissions = 23106L,
+    drugitems = 4907269L,
+    freetextitems = 651248L,
+    listitems = 30744065L,
+    numericitems = 977625612L,
+    procedureorderitems = 2188626L,
+    processitems = 256715L
+  )
+
+  part <- list(
+    listitems = list(col = "itemid", breaks = 12290L),
+    numericitems = list(col = "itemid", breaks = c(
+       6641L,  6642L,  6643L,  6664L,  6666L,  6667L,  6669L,  6672L,  6673L,
+       6675L,  6707L,  6709L,  8874L, 12270L, 12275L, 12278L, 12281L, 12286L,
+      12303L, 12561L, 12576L, 12804L, 14841L)
+    )
+  )
+
+  tables <- Map(list, files = setNames(paste0(tables, ".csv"), tables),
+                 defaults = defaults[tables], num_rows = n_row[tables],
+                 cols = cols[tables])
+
+  tables[names(part)] <- Map(`[[<-`, tables[names(part)], "partitioning", part)
+
+  tables
+}
+
 pkg_dir <- rprojroot::find_root(rprojroot::is_r_package)
 cfg_dir <- file.path(pkg_dir, "inst", "extdata", "config")
 
@@ -668,17 +874,23 @@ cfg <- list(
   ),
   list(
     name = "hirid",
-    url = "https://physionet.org/files/hirid/1.0",
-    unzip = c(
-      "reference_data.tar.gz",
-      "raw_stage/observation_tables_csv.tar.gz",
-      "raw_stage/pharma_records_csv.tar.gz"
-    ),
+    url = "https://physionet.org/files/hirid/1.1.1",
     id_cfg = list(
       icustay = list(id = "patientid", position = 1L, start = "admissiontime",
                      table = "general")
     ),
     tables = hirid_tbl_cfg()
+  ),
+  list(
+    name = "aumc",
+    id_cfg = list(
+      patient = list(id = "patientid", position = 1L,
+                     start = "firstadmittedat", end = "dateofdeath",
+                     table = "admissions"),
+      icustay = list(id = "admissionid", position = 2L, start = "admittedat",
+                     end = "dischargedat", table = "admissions")
+    ),
+    tables = aumc_tbl_cfg()
   )
 )
 
@@ -686,19 +898,19 @@ ricu::set_config(cfg, "data-sources", cfg_dir)
 
 cfg <- list(
   list(
-    name = "eicu_demo",
-    class_prefix = c("eicu_demo", "eicu"),
+    name = "eicu_test",
+    class_prefix = c("eicu_test", "eicu_demo", "eicu"),
     id_cfg = eicu_id_cfg,
     tables = lapply(eicu_demo_tbls, as_minimal_tbl_spec)
   ),
   list(
-    name = "mimic_demo",
-    class_prefix = c("mimic_demo", "mimic"),
+    name = "mimic_test",
+    class_prefix = c("mimic_test", "mimic_demo", "mimic"),
     id_cfg = mimic_id_cfg,
     tables = lapply(mimic_demo_tbls, as_minimal_tbl_spec)
   )
 )
 
-ricu::set_config(cfg, "demo-sources", cfg_dir)
+ricu::set_config(cfg, "data-sources", file.path(pkg_dir, "inst", "testdata"))
 
 devtools::install(pkg_dir)
